-- to do
-- add a filter for annotations - high priority
-- change annotationWindow so that new annotations are inserted in the list alphabetically, have to remember to remove event handlers when moving list entries...
-- -- this isn't high priorit as they will auto sort on the next load
-- foreign language support, need french and german translations of String.lua, especially the EXACT map names from the client
-- add a drop down to add additional maps... long term goal for non-Moors version
-- add icons for all of the standard map annotations

import "GaranStuff.MoorMap.Class";
import "GaranStuff.MoorMap.VindarPatch";
-- This is the Load/Save patch which accounts for European formats, written by Vindar
-- load resource strings - this MUST occur before any windows that use strings get loaded or there will be no defaults
import "GaranStuff.MoorMap.Strings";
import "GaranStuff.MoorMap.Table";
import "Turbine";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Lotro";
import "GaranStuff.MoorMap.DropDownList";
import "GaranStuff.MoorMap.OptionsWindow";
import "GaranStuff.MoorMap.AnnotationsWindow";
import "GaranStuff.MoorMap.MinimizedWindow";
import "GaranStuff.MoorMap.PopUpDialog";

language=1;

colorGold=Turbine.UI.Color(.80,.60,.1);
colorSilver=Turbine.UI.Color(.75,.75,.75);
colorDarkGrey=Turbine.UI.Color(.1,.1,.1);

trimColor=Turbine.UI.Color(.2,.5,.9);
fontColor=Turbine.UI.Color(1,.9,.5);
backColor=Turbine.UI.Color(.05,.05,.05);
headingsColor=Turbine.UI.Color(1,1,1);
listTextColor=Turbine.UI.Color(1,1,1);

fontFace=Turbine.UI.Lotro.Font.TrajanPro16;
dataLoadError=false;
saveSettings=true;
priorDataVersion=nil;

localPlayer=Turbine.Gameplay.LocalPlayer.GetInstance();
currentCharacterName=string.gsub(localPlayer:GetName(),"~","_");
undefinedCategoryIndex=1;

annotations={};
-- all annotations have the form:{Map ID, X, Y, Z, H, V, Icon (either resourceID or path), Type, TextID, Description}
-- Annotation Type follows the Map Filter types plus the Creep specific "Map In"
-- TextID is only valid for the defaults and will allow using the appropriately translated text value from Resource
mapInfo={};
-- mapInfo is in the form {MapName,Height,Width,HFactor,HOffset,WFactor,WOffset}
-- HFactor and HOffset are used in the conversion from map coordinates to pixels: X=Height*HFactor+HOffset will convert coordinate "Height" to pixel "X"

iconInfo={};

function UnloadPlugin()
	mapWindow:SetPluginUnloading(true);
	if saveSettings then -- if there were errors loading data, the user can choose to abort saving as it would destroy their prior data
		SaveData();
	end

	-- have to remove all shell commands before unloading or we get a crash to desktop :(
	mapWindow:RemoveShellCommands();
	mapWindow.MouseClick=nil;

	Turbine.Shell.WriteLine("MoorMap "..Plugins["MoorMap"]:GetVersion()..Resource[language][10]);
end

function SaveData()
	-- save the character scope settings
	local settings={};
	local tmpIndex;
	settings[#settings+1]={"MapTop",mapWindow:GetTop()};
	settings[#settings+1]={"MapLeft",mapWindow:GetLeft()};
	settings[#settings+1]={"MapHeight",mapWindow:GetHeight()};
	settings[#settings+1]={"MapWidth",mapWindow:GetWidth()};
	settings[#settings+1]={"MapOpacity",mapWindow.Map:GetOpacity()};
	settings[#settings+1]={"DataVersion",Plugins["MoorMap"]:GetVersion()};

	settings[#settings+1]={"IconTop",minWindow:GetTop()};
	settings[#settings+1]={"IconLeft",minWindow:GetLeft()};

	settings[#settings+1]={"fontColor",fontColor};
	settings[#settings+1]={"trimColor",trimColor};
	settings[#settings+1]={"backColor",backColor};
	settings[#settings+1]={"headingsColor",headingsColor};
	settings[#settings+1]={"listTextColor",listTextColor};
	settings[#settings+1]={"loadMinimized",mapWindow:GetLoadMinimized()};
	settings[#settings+1]={"language",language};
	settings[#settings+1]={"showIconWhenMinimized",mapWindow:GetShowIconWhenMinimized()};

	PatchDataSave( Turbine.DataScope.Account, "MoorMapSettings", settings );

	PatchDataSave( Turbine.DataScope.Account, "MM_IconInfo", iconInfo);
	PatchDataSave( Turbine.DataScope.Account, "MM_MapInfo", mapInfo);
	PatchDataSave( Turbine.DataScope.Account, "MM_Anotations", annotations);
end

function LoadData()
	-- try to load saved annotations
--**** loading is temporarily disabled until the defaults are complete
	iconInfo=PatchDataLoad( Turbine.DataScope.Account, "MM_IconInfo");
	annotations=PatchDataLoad( Turbine.DataScope.Account, "MM_Anotations");
	mapInfo=PatchDataLoad( Turbine.DataScope.Account, "MM_MapInfo");
	if type(annotations)~="table" or #annotations==0 then
		-- if this fails, then load the defaults from defaults.lua
		import "GaranStuff.MoorMap.Defaults";
		loadDefaults();
		priorDataVersion=Plugins["MoorMap"]:GetVersion();
	end
end


MMWindow = class( Turbine.UI.Window );

function MMWindow:Constructor()
	Turbine.UI.Window.Constructor( self );
	self:SetSize(1024,768);
	self.Loaded=false;
	-- load the settings and set the window position if possible, otherwise default to center
	local initHeight=768;
	local initWidth=1024;
	local initLeft=(Turbine.UI.Display:GetWidth() - initWidth) / 2;
	local initTop=(Turbine.UI.Display:GetHeight() - initHeight) / 2;
	local tmpIndex;
	local mapOpacity=1;

	self.CurrentMapID=1; -- currently we only support the Ettenmoors map
	self.NeedsRefresh=false;
	self.PopUp=nil;

	self.PluginUnloading=false;
	self.Resizing=false;
	self.MoveX=-1;
	self.MoveY=-1;
	self.LoadMinimized=true;
	self.ShowIconWhenMinimized=true;
	tmpSettings=PatchDataLoad(Turbine.DataScope.Account, "MoorMapSettings");
	if (tmpSettings~=nil) then
		if tostring(type(tmpSettings))=="table" then
			for index=1,#tmpSettings do
				if tostring(type(tmpSettings[index]))=="table" then
					if (tmpSettings[index][1]=="MapTop") then
						initTop=tmpSettings[index][2];
					elseif (tmpSettings[index][1]=="MapLeft") then
						initLeft=tmpSettings[index][2];
					elseif (tmpSettings[index][1]=="MapHeight") then
						initHeight=tmpSettings[index][2];
					elseif (tmpSettings[index][1]=="MapWidth") then
						initWidth=tmpSettings[index][2];
					elseif (tmpSettings[index][1]=="MapOpacity") then
						mapOpacity=tmpSettings[index][2];
					elseif (tmpSettings[index][1]=="trimColor") then
						trimColor=Turbine.UI.Color(tmpSettings[index][2].R,tmpSettings[index][2].G,tmpSettings[index][2].B);
					elseif (tmpSettings[index][1]=="fontColor") then
						fontColor=Turbine.UI.Color(tmpSettings[index][2].R,tmpSettings[index][2].G,tmpSettings[index][2].B);
					elseif (tmpSettings[index][1]=="backColor") then
						backColor=Turbine.UI.Color(tmpSettings[index][2].R,tmpSettings[index][2].G,tmpSettings[index][2].B);
					elseif (tmpSettings[index][1]=="headingsColor") then
						headingsColor=Turbine.UI.Color(tmpSettings[index][2].R,tmpSettings[index][2].G,tmpSettings[index][2].B);
					elseif (tmpSettings[index][1]=="listTextColor") then
						listTextColor=Turbine.UI.Color(tmpSettings[index][2].R,tmpSettings[index][2].G,tmpSettings[index][2].B);
					elseif (tmpSettings[index][1]=="loadMinimized") then
						self.LoadMinimized=tmpSettings[index][2];
					elseif (tmpSettings[index][1]=="language") then
						if tonumber(tmpSettings[index][2])~=nil then
							language=tonumber(tmpSettings[index][2]);
						end
					elseif (tmpSettings[index][1]=="showIconWhenMinimized") then
						self.ShowIconWhenMinimized=tmpSettings[index][2];
					elseif (tmpSettings[index][1]=="DataVersion") then
						priorDataVersion=tmpSettings[index][2];
					end
				end
			end
		else
			-- data was unreadable, we need to display a message to the user that the data could not be read
			dataLoadError=true
		end
		-- account for changed resolution
		if initLeft>(Turbine.UI.Display.GetWidth()-initWidth) then initLeft=Turbine.UI.Display.GetWidth()-initWidth end;
		if initLeft<0 then initLeft=0 end;
		if initWidth>(Turbine.UI.Display.GetWidth()-initLeft) then initWidth=Turbine.UI.Display.GetWidth()-initLeft end;
		if initTop>(Turbine.UI.Display.GetHeight()-initHeight) then initTop=Turbine.UI.Display.GetHeight()-initHeight end;
		if initTop<0 then initTop=0 end;
		if initHeight>(Turbine.UI.Display.GetHeight()-initTop) then initTop=Turbine.UI.Display.GetHeight()-initHeight end;
	end
	self:SetPosition( initLeft, initTop );
	self:SetHeight(initHeight);
	self:SetWidth(initWidth);
	self:SetVisible(false);

	self.Map=Turbine.UI.Control();
	self.Map:SetParent(self);
	self.Map:SetMouseVisible(false);
	self.Map:SetSize(mapInfo[self.CurrentMapID][4],mapInfo[self.CurrentMapID][5]);
	self.Map:SetBackground(mapInfo[self.CurrentMapID][3]);
	self.Map:SetStretchMode(1);
	self.Map:SetSize(self:GetSize());
	self.Map:SetOpacity(mapOpacity);

	self.Map.OptionsButton=Turbine.UI.Control();
	self.Map.OptionsButton:SetParent(self.Map);
	self.Map.OptionsButton:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	self.Map.OptionsButton:SetBackground("GaranStuff/MoorMap/Resources/options.tga");
	self.Map.OptionsButton:SetSize(32,32);
	self.Map.OptionsButton:SetTop(4);
	self.Map.OptionsButton:SetLeft(18);
	self.Map.OptionsButton.MouseEnter=function()
		self.Map.OptionsButton:SetBackground("GaranStuff/MoorMap/Resources/options_hover.tga");
	end
	self.Map.OptionsButton.MouseLeave=function()
		self.Map.OptionsButton:SetBackground("GaranStuff/MoorMap/Resources/options.tga");
	end
	self.Map.OptionsButton.MouseClick=function()
		optWindow:SetVisible(true);
	end

	self.Map.AnnotationsButton=Turbine.UI.Control();
	self.Map.AnnotationsButton:SetParent(self.Map);
	self.Map.AnnotationsButton:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	self.Map.AnnotationsButton:SetBackground("GaranStuff/MoorMap/Resources/annotations.tga");
	self.Map.AnnotationsButton:SetSize(32,32);
	self.Map.AnnotationsButton:SetTop(4);
	self.Map.AnnotationsButton:SetLeft(self.Map.OptionsButton:GetLeft()+self.Map.OptionsButton:GetWidth()+5);
	self.Map.AnnotationsButton.MouseEnter=function()
		self.Map.AnnotationsButton:SetBackground("GaranStuff/MoorMap/Resources/annotations_hover.tga");
	end
	self.Map.AnnotationsButton.MouseLeave = function()
		self.Map.AnnotationsButton:SetBackground("GaranStuff/MoorMap/Resources/annotations.tga");
	end
	self.Map.AnnotationsButton.MouseClick=function()
		annotWindow:SetVisible(true);
	end

	self.Map.SearchButton=Turbine.UI.Control();
	self.Map.SearchButton:SetParent(self.Map);
	self.Map.SearchButton:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	self.Map.SearchButton:SetBackground("GaranStuff/MoorMap/Resources/Search.tga");
	self.Map.SearchButton:SetSize(32,32);
	self.Map.SearchButton:SetTop(4);
	self.Map.SearchButton:SetLeft(self.Map.AnnotationsButton:GetLeft()+self.Map.AnnotationsButton:GetWidth()+5);
	self.Map.SearchButton.MouseEnter=function()
		self.Map.SearchButton:SetBackground("GaranStuff/MoorMap/Resources/Search_hover.tga");
	end
	self.Map.SearchButton.MouseLeave=function()
		self.Map.SearchButton:SetBackground("GaranStuff/MoorMap/Resources/Search.tga");
	end
	self.Map.SearchButton.MouseClick=function()
		self.Map.IconList:SetVisible(true);
		self.Map.IconSearch:SetVisible(true);
		self.Map.IconSearch:SetWantsUpdates(true);
	end

	self.Map.NSCoord=Turbine.UI.Label();
	self.Map.NSCoord:SetParent(self.Map);
	self.Map.NSCoord:SetSize(40,18);
	self.Map.NSCoord:SetPosition(self.Map.SearchButton:GetLeft()+self.Map.SearchButton:GetWidth()+5,14);
	self.Map.NSCoord:SetForeColor(fontColor);
	self.Map.NSCoord:SetBackColor(backColor);
	self.Map.NSCoord:SetFont(fontFace);
	self.Map.NSCoord:SetText("");

	self.Map.EWCoord=Turbine.UI.Label();
	self.Map.EWCoord:SetParent(self.Map);
	self.Map.EWCoord:SetSize(40,18);
	self.Map.EWCoord:SetPosition(self.Map.NSCoord:GetLeft()+self.Map.NSCoord:GetWidth()+5,14);
	self.Map.EWCoord:SetForeColor(fontColor);
	self.Map.EWCoord:SetBackColor(backColor);
	self.Map.EWCoord:SetFont(fontFace);
	self.Map.EWCoord:SetText("");

	self.Map.CloseButton=Turbine.UI.Control();
	self.Map.CloseButton:SetParent(self.Map);
	self.Map.CloseButton:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	self.Map.CloseButton:SetBackground("GaranStuff/MoorMap/Resources/Close.tga");
	self.Map.CloseButton:SetSize(16,16);
	self.Map.CloseButton:SetTop(4);
	self.Map.CloseButton:SetLeft(self.Map:GetWidth()-20);
	self.Map.CloseButton.MouseEnter = function()
		self.Map.CloseButton:SetBackground("GaranStuff/MoorMap/Resources/Close_hover.tga");
	end
	self.Map.CloseButton.MouseLeave = function()
		self.Map.CloseButton:SetBackground("GaranStuff/MoorMap/Resources/Close.tga");
	end
	self.Map.CloseButton.MouseClick=function()
		mapWindow:SetVisible(false);
	end

	self.Map.Icons={}
	self.Map.MapPoints={}

	self.Map.IconSearch=Turbine.UI.Lotro.TextBox();
	self.Map.IconSearch:SetParent(self.Map);
	self.Map.IconSearch:SetSize(390,20);
	self.Map.IconSearch:SetFont(fontFace);
	self.Map.IconSearch:SetPosition(self.Map:GetWidth()-410,10);
	self.Map.IconSearch:SetVisible(false);
	self.Map.IconSearch:SetText("");
	self.Map.IconSearch.OldText="";
	self.Map.IconSearch.Update=function(sender)
		if mapWindow.Map.IconSearch:GetText()~=mapWindow.Map.IconSearch.OldText then
			mapWindow.Map.IconSearch.OldText=mapWindow.Map.IconSearch:GetText();
			-- filter the results by search string...
			local tmpIndex;
			for tmpIndex=1,mapWindow.Map.IconList:GetItemCount() do
				if mapWindow.Map.IconSearch:GetText()=="" then
					mapWindow.Map.IconList:GetItem(tmpIndex):SetHeight(mapWindow.Map.IconList:GetItem(tmpIndex).NormalHeight);
				else
					if mapWindow.Map.IconList:GetItem(tmpIndex).AnnotationIndex~=0 then
						if string.find(string.lower(mapWindow.Map.IconList:GetItem(tmpIndex):GetText()),string.lower(mapWindow.Map.IconSearch:GetText()),nil,true)~=nil then
							mapWindow.Map.IconList:GetItem(tmpIndex):SetHeight(mapWindow.Map.IconList:GetItem(tmpIndex).NormalHeight);
						else
							mapWindow.Map.IconList:GetItem(tmpIndex):SetHeight(0);
						end
					end
				end
			end
		end
	end
	self.Map.IconList=Turbine.UI.ListBox();
	self.Map.IconList:SetParent(self.Map);
	self.Map.IconList:SetSize(400,self.Map:GetHeight()-45);
	self.Map.IconList:SetPosition(self.Map:GetWidth()-410,35);
	self.Map.IconList:SetVisible(false);
	self.Map.IconSearch:SetWantsUpdates(false);
	self.Map.IconList.VScroll=Turbine.UI.Lotro.ScrollBar();
	self.Map.IconList.VScroll:SetOrientation(Turbine.UI.Orientation.Vertical);
	self.Map.IconList.VScroll:SetParent(self.Map.IconList);
	self.Map.IconList.VScroll:SetBackColor(backColor);
	self.Map.IconList.VScroll:SetPosition(self.Map.IconList:GetWidth()-12,0);
	self.Map.IconList.VScroll:SetWidth(12);
	self.Map.IconList.VScroll:SetHeight(self.Map.IconList:GetHeight());
	self.Map.IconList:SetVerticalScrollBar(self.Map.IconList.VScroll);
	self.Map.IconList.VScroll:SetVisible(false);

	-- I temporarily use 7 controls to cover each map point in a 14x16 area because the mouse events for ItemControls only fire on their borders. As soon as he mouse events are fixed, I will eliminate the excess controls.
	self.Map.MapPoints2={}
	self.Map.MapPoints3={}
	self.Map.MapPoints4={}
	self.Map.MapPoints5={}
	self.Map.MapPoints6={}
	self.Map.MapPoints7={}

	self.Map.Ping=Turbine.UI.Control();
	self.Map.Ping:SetParent(self.Map);
	self.Map.Ping.State=0;
	self.Map.Ping:SetVisible(false);
	self.Map.Ping:SetSize(23,23);
	self.Map.Ping:SetPosition(512,512);
	self.Map.Ping:SetWantsUpdates(false);
	self.Map.Ping.Counter=1;
	self.Map.Ping.MaxCounter=2;
	self.Map.Ping:SetBlendMode(Turbine.UI.BlendMode.Overlay)
	self.Map.Ping:SetBackground("GaranStuff/MoorMap/Resources/Ping1.tga");
	self.Map.Ping.MouseClick=function(sender,args)
		mapWindow.Map.Ping.State=0; -- it will auto hide itself on next tick
	end
	self.Map.Ping.Update=function(sender)
		if self.Map.Ping.State==0 then
			self.Map.Ping:SetWantsUpdates(false);
			self.Map.Ping:SetVisible(false);
			self.Map.Ping.Counter=0;
			self.Map.Ping:SetBackground("GaranStuff/MoorMap/Resources/Ping1.tga");
		elseif self.Map.Ping.State==1 then
			self.Map.Ping.Counter=self.Map.Ping.Counter+1;
			if self.Map.Ping.Counter>self.Map.Ping.MaxCounter then
				self.Map.Ping.Counter=0;
				self.Map.Ping.State=2;
				self.Map.Ping:SetBackground("GaranStuff/MoorMap/Resources/Ping2.tga");
			end
		elseif self.Map.Ping.State==2 then
			self.Map.Ping.Counter=self.Map.Ping.Counter+1;
			if self.Map.Ping.Counter>self.Map.Ping.MaxCounter then
				self.Map.Ping.Counter=0;
				self.Map.Ping.State=3;
				self.Map.Ping:SetBackground("GaranStuff/MoorMap/Resources/Ping3.tga");
			end
		elseif self.Map.Ping.State==3 then
			self.Map.Ping.Counter=self.Map.Ping.Counter+1;
			if self.Map.Ping.Counter>self.Map.Ping.MaxCounter then
				self.Map.Ping.Counter=0;
				self.Map.Ping.State=4;
				self.Map.Ping:SetBackground("GaranStuff/MoorMap/Resources/Ping4.tga");
			end
		elseif self.Map.Ping.State==4 then
			self.Map.Ping.Counter=self.Map.Ping.Counter+1;
			if self.Map.Ping.Counter>self.Map.Ping.MaxCounter then
				self.Map.Ping.Counter=0;
				self.Map.Ping.State=5;
				self.Map.Ping:SetBackground("GaranStuff/MoorMap/Resources/Ping5.tga");
			end
		elseif self.Map.Ping.State==5 then
			self.Map.Ping.Counter=self.Map.Ping.Counter+1;
			if self.Map.Ping.Counter>self.Map.Ping.MaxCounter then
				self.Map.Ping.Counter=0;
				self.Map.Ping.State=6;
				self.Map.Ping:SetBackground("GaranStuff/MoorMap/Resources/Ping6.tga");
			end
		elseif self.Map.Ping.State==6 then
			self.Map.Ping.Counter=self.Map.Ping.Counter+1;
			if self.Map.Ping.Counter>self.Map.Ping.MaxCounter then
				self.Map.Ping.Counter=0;
				self.Map.Ping.State=7;
				self.Map.Ping:SetBackground("GaranStuff/MoorMap/Resources/Ping7.tga");
			end
		elseif self.Map.Ping.State==7 then
			self.Map.Ping.Counter=self.Map.Ping.Counter+1;
			if self.Map.Ping.Counter>self.Map.Ping.MaxCounter then
				self.Map.Ping.Counter=0;
				self.Map.Ping.State=3;
				self.Map.Ping:SetBackground("GaranStuff/MoorMap/Resources/Ping3.tga");
			end
		end
	end

	self.Map.HoverInfo=Turbine.UI.ContextMenu();
	self.Map.HoverInfo:GetItems():Add(Turbine.UI.MenuItem("Name"));
	self.Map.HoverInfo:GetItems():Get(1):SetEnabled(false);
	self.Map.HoverInfo:GetItems():Add(Turbine.UI.MenuItem("Coord"));
	self.Map.HoverInfo:GetItems():Get(2):SetEnabled(false);
	self.Map.HoverInfo:GetItems():Add(Turbine.UI.MenuItem("X:Y"));
	self.Map.HoverInfo:GetItems():Get(3):SetEnabled(false);
	self.Map.HoverInfo:GetItems():Add(Turbine.UI.MenuItem("Edit"));
	self.Map.HoverInfo:GetItems():Get(4):SetEnabled(true);
	self.Map.HoverInfo:GetItems():Get(4).Click=function()
		annotWindow:SetVisible(true);
	end

	self:SetText(Resource[language][2]);

	self.SetPluginUnloading = function(value)
		self.PluginUnloading=value;
	end
	self.VisibleChanged=function()
		if (not self.PluginUnloading) then
			if not self:IsVisible() then
				minWindow:SetVisible(true);
				if optWindow~=nil then optWindow:SetVisible(false) end;
				if annotWindow~=nil then annotWindow:SetVisible(false) end;
				if mapWindow.PopUp~=nil then
					mapWindow.PopUp:SetVisible(false);
					mapWindow.PopUp=nil; -- dismiss the dialog
				end
			else
				-- refresh anything that needs refreshing when map is visible
				RefreshMap();
			end
		end
	end

	self.AddAnnotationMenu=Turbine.UI.ContextMenu();
	self.AddAnnotationMenu:GetItems():Add(Turbine.UI.MenuItem(Resource[language][154]));
	self.AddAnnotationMenu:GetItems():Get(1).Click=function()
		annotWindow.AnnotationList:SetSelectedIndex(1);
		annotWindow.TypeList:SetSelectedIndex(6);
		annotWindow.NameText:SetText("");
		annotWindow.DescriptionText:SetText("");
		annotWindow:SetVisible(true);
	end

	self.MovingIcon=Turbine.UI.Control();
	self.MovingIcon:SetParent(self);
	self.MovingIcon:SetSize(32,32);
	self.MovingIcon:SetStretchMode(1); -- allows it to display past the parent's bounds
	self.MovingIcon:SetBackground(0x410081c0)
	-- for some reason, when StretchMode=1, setting the control's position or size AFTER loading the resource background makes it appear... WEIRD
	self.MovingIcon:SetPosition(self:GetWidth()/2-15,self:GetHeight()-21);
	self.MovingIcon:SetVisible(false);

	self.KeyDown=function(sender, args)
--Turbine.Shell.WriteLine("KeyDown Action="..args.Action..", .Alt="..tostring(args.Alt)..", .Control="..tostring(args.Control)..", .Shift="..tostring(args.Shift))
		if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then
			mapWindow:SetVisible(false);
			minWindow:SetVisible(true);
		end
		if ( args.Action == 268435635 ) then -- toggle HUD
			mapWindow:SetVisible( false );
			minWindow:SetVisible( true ); -- this is harmless if the "ShowIconWhenMinimized" option is turned off
			-- if turbine ever exposes the UI state, we can force the minimized icon not to display when the HUD is off
		end
	end

	self.SetHeight=function(sender,height)
		if height<297 then height=297 end;
		Turbine.UI.Window.SetHeight(self,height);
		-- now move/resize all the controls
		self.MovingIcon:SetTop(height-21);
		self.Map:SetSize(self:GetWidth(),height); -- when using stretchmode=1, you HAVE to use setSize or the other parameter goes back to "actual" value
	end

	self.SetSize=function(sender, width, height)
		Turbine.UI.Window.SetSize(self,width,height);
		if self.Map ~=nil then
			self.Map:SetSize(width,height);
		end
	end

	self.SetWidth=function(sender,width)
		-- in minimal mode, the width is already padded 10 pixels to account for normal mode, so just leave it.
		if width<414 then width=414 end;
		Turbine.UI.Window.SetWidth(self,width);
		self.MovingIcon:SetLeft(width-21);
		self.Map:SetSize(width, self:GetHeight());
	end

	self.MouseDown=function(sender,args)
		if (args.Y>self:GetHeight()-15) and (args.X>self:GetWidth()-15) then
			self.MoveX=args.X;
			self.MoveY=args.Y;
			self.MovingIcon:SetLeft(args.X-12);
			self.MovingIcon:SetTop(args.Y-12);
			self.MovingIcon:SetBackground(0x41007e20)
			self.MovingIcon:SetVisible(true);
		elseif (args.Y>self:GetHeight()-12) then
			self.MoveY=args.Y;
			self.MovingIcon:SetLeft(args.X-22);
			self.MovingIcon:SetTop(args.Y-12);
			self.MovingIcon:SetBackground(0x410081c0)
			self.MovingIcon:SetVisible(true);
		elseif (args.X>self:GetWidth()-12) then
			self.MoveX=args.X;
			self.MovingIcon:SetLeft(args.X-12);
			self.MovingIcon:SetTop(args.Y-22);
			self.MovingIcon:SetBackground(0x410081bf)
			self.MovingIcon:SetVisible(true);
		elseif (args.Y<8) then
			self.MoveX=args.X;
			self.MoveY=args.Y;
			self.MovingIcon:SetLeft(args.X-12);
			self.MovingIcon:SetTop(args.Y-12);
			self.MovingIcon:SetSize(16,16);
			self.MovingIcon:SetBackground(0x41007e0c)
			self.MovingIcon:SetVisible(true);
		else
			self.MoveX=-1;
			self.MoveY=-1;
		end
		if self.PopUp~=nil then
			self.PopUp:SetVisible(false);
			self.PopUp=nil; -- dismiss the dialog
		end
	end

	self.MouseMove=function(sender,args)
		if (self.MoveY>-1 and self.MoveY<8) then
			if (args.X~=self.MoveX or args.Y~= self.MoveY) then
				self.Moving=true;
				local newLeft=self:GetLeft()-(self.MoveX-args.X)
				local newTop=self:GetTop()-(self.MoveY-args.Y)
				if newLeft<0 then newLeft=0 end;
				if newLeft>(Turbine.UI.Display.GetWidth()-self:GetWidth()) then newLeft=Turbine.UI.Display.GetWidth()-self:GetWidth() end;
				if newTop<0 then newTop=0 end;
				if newTop>(Turbine.UI.Display.GetHeight()-self:GetHeight()) then newTop=Turbine.UI.Display.GetHeight()-self:GetHeight() end;
				self:SetPosition(newLeft,newTop);
			end
		else		
			if (self.MoveY>-1) then
				if args.Y~= self.MoveY then
					self.Resizing=true;
					local newHeight=self:GetHeight()-(self.MoveY-args.Y);
					if newHeight<215 then newHeight=215 end;
					if newHeight>(Turbine.UI.Display.GetHeight()-self:GetTop()) then newHeight=Turbine.UI.Display.GetHeight()-self:GetTop() end;
					local newX=args.X-22;
					if newX<-13 then newX=-13 end
					if newX>(self:GetWidth()-18) then newX=self:GetWidth()-18 end
					self.MovingIcon:SetLeft(newX);
					self:SetHeight(newHeight);
					self.MoveY=args.Y;
				end
			end
			if (self.MoveX>-1) then
				if args.X~= self.MoveX then
					self.Resizing=true;
					local newWidth=self:GetWidth()-(self.MoveX-args.X);
					if newWidth<414 then newWidth=414 end;
					if newWidth>(Turbine.UI.Display.GetWidth()-self:GetLeft()) then newWidth=Turbine.UI.Display.GetWidth()-self:GetLeft() end;
					local newY=args.Y-22;
					if newY<-13 then newY=-13 end
					if newY>(self:GetHeight()-18) then newY=self:GetHeight()-18 end
					self.MovingIcon:SetTop(newY);
					self:SetWidth(newWidth);
					self.MoveX=args.X;
				end
			end
		end
		if not self.Moving and not self.Resizing then
			-- update location display
--looks like we need to scale here
			local XScale=mapInfo[mapWindow.CurrentMapID][4]/self:GetWidth();
			local YScale=mapInfo[mapWindow.CurrentMapID][5]/self:GetHeight();
			local EW=math.floor((args.X*XScale-mapInfo[mapWindow.CurrentMapID][7])/mapInfo[mapWindow.CurrentMapID][6]*10)/10;
			local NS=math.floor((args.Y*YScale-mapInfo[mapWindow.CurrentMapID][9])/mapInfo[mapWindow.CurrentMapID][8]*10)/10;
			if EW<0 then
				self.Map.EWCoord:SetText(tostring(0-EW).."W");
			else
				self.Map.EWCoord:SetText(tostring(EW).."W");
			end
			if NS<0 then
				self.Map.NSCoord:SetText(tostring(0-NS).."S");
			else
				self.Map.NSCoord:SetText(tostring(NS).."S");
			end
		end
	end

	self.MouseUp=function(sender,args)
		self.MovingIcon:SetVisible(false);
		self.MoveX=-1;
		self.MoveY=-1;
		self.Moving=false;
		self.Resizing=false;
	end

	self.MouseClick=function(sender,args)
		self.Map.IconList:SetVisible(false);
		self.Map.IconSearch:SetVisible(false);
		self.Map.IconSearch:SetWantsUpdates(false);
		if not self.Moving and not self.Resizing and args.Button==2 then
			-- user right clicked on map, popup the "Add Annotation" option
			local XScale=mapInfo[mapWindow.CurrentMapID][4]/self:GetWidth();
			local YScale=mapInfo[mapWindow.CurrentMapID][5]/self:GetHeight();
			annotWindow.EWCoordText:SetText(tostring(math.floor((args.X*XScale-mapInfo[mapWindow.CurrentMapID][7])/mapInfo[mapWindow.CurrentMapID][6]*10)/10));
			annotWindow.NSCoordText:SetText(tostring(math.floor((args.Y*YScale-mapInfo[mapWindow.CurrentMapID][9])/mapInfo[mapWindow.CurrentMapID][8]*10)/10));
			self.AddAnnotationMenu:ShowMenu();
		end
	end

-- set ZOrder so that the drop downs can cover other items, etc
	self.Map:SetZOrder(1);
	self.Map.OptionsButton:SetZOrder(5);
	self.Map.AnnotationsButton:SetZOrder(7);
	self.Map.SearchButton:SetZOrder(10);
	self.MovingIcon:SetZOrder(15);

	self:SetWantsKeyEvents(true);
	self:SetWantsUpdates(true)

	self.RemoveShellCommands=function(sender,args)
		Turbine.Shell.RemoveCommand(self.shellCmd)
	end
	self.shellCmd=Turbine.ShellCommand();
	local numberOfCommandsRegistered = Turbine.Shell.AddCommand("MoorMap",self.shellCmd);

	self.shellCmd.Execute = function(sender, cmd, args)
		if (string.lower(cmd) == "moormap") then
			if (string.lower(args)=="show") then
				minWindow:SetVisible(false);
				self:SetVisible(true);
			elseif (string.lower(args)=="hide") then
				mapWindow:SetVisible(false);
				minWindow:SetVisible(true);
			elseif (string.lower(args)=="toggle") then
				mapWindow:SetVisible(not mapWindow:IsVisible());
				minWindow:SetVisible(not minWindow:IsVisible());
			else
				Turbine.Shell.WriteLine("usage:");
				Turbine.Shell.WriteLine("  /MoorMap show");
				Turbine.Shell.WriteLine("       Displays the MoorMap map.");
				Turbine.Shell.WriteLine("  /MoorMap hide");
				Turbine.Shell.WriteLine("       Hides the MoorMap map.");
				Turbine.Shell.WriteLine("  /MoorMap toggle");
				Turbine.Shell.WriteLine("       Toggles the MoorMap map (shows it if it was hidden, hides it if it was showing).");
			end
		end
	end
	self.shellCmd.GetHelp = function(sender, cmd)
		return("usage:\n  /MoorMap show\n       Displays the MoorMap map.\n  /MoorMap hide\n       Hides the MoorMap map.\n  /MoorMap toggle\n       Toggles the MoorMap map (shows it if it was hidden, hides it if it was showing).");
	end

	self.shellCmd.GetShortHelp = function(sender, cmd)
		return("MoorMap - usage: /MoorMap show | hide | toggle");
	end
	self.Closing=function()
	end

	self.GetShowIconWhenMinimized=function()
		return self.ShowIconWhenMinimized;
	end
	self.SetShowIconWhenMinimized=function(sender,value)
		self.ShowIconWhenMinimized=value;
		if value then
			minWindow:SetSize(32,32);
		else
			minWindow:SetSize(0,0);
		end
	end

	self.SetTrimColor=function()
	end
	self.SetBackColor=function()
		self.Map.NSCoord:SetBackColor(backColor);
		self.Map.EWCoord:SetBackColor(backColor);
	end
	self.SetListTextColor=function()
	end
	self.SetFontColor=function()
		self.Map.NSCoord:SetForeColor(fontColor);
		self.Map.EWCoord:SetForeColor(fontColor);
	end
	self.SetHeadingsColor=function()
	end
	self.SetLanguage=function()
		self:SetText(Resource[language][2]);
		self.AddAnnotationMenu:GetItems():Get(1):SetText(Resource[language][154]);
		RefreshMap();
	end
end

function RefreshMap(MapID)
	if MapID==nil then MapID=mapWindow.CurrentMapID end
	if MapID~=mapWindow.CurrentMapID then
		-- if we ever start supporting other maps, THIS is where we would load the background and annotations for a new map
		mapWindow.CurrentMapID=MapID;
		mapWindow.Map:SetSize(mapInfo[self.CurrentMapID][4],mapInfo[self.CurrentMapID][5]);
		mapWindow.Map:SetBackground(mapInfo[self.CurrentMapID][3]);
		mapWindow.Map:SetStretchMode(1);
		mapWindow.Map:SetSize(self:GetSize());
		mapWindow.Map.CloseButton:SetLeft(mapWindow.Map:GetWidth()-20);
		mapWindow.Map.IconList:SetHeight(self.Map:GetHeight()-45);
		mapWindow.Map.IconList:SetLeft(self.Map:GetWidth()-210);
		mapWindow.Map.IconList.VScroll:SetHeight(self.Map.IconList:GetHeight());
		mapWindow.Map.IconList:SetVisible(false);
		mapWindow.Map.IconSearch:SetVisible(false);
		mapWindow.Map.IconSearch:SetWantsUpdates(false);
	end
	-- clear out the existing search list
	while mapWindow.Map.IconList:GetItemCount()>0 do
		-- clear the event handler
		mapWindow.Map.IconList:GetItem(1).MouseClick=nil;
		mapWindow.Map.IconList:RemoveItemAt(1);
	end

	LoadMapsIns(); -- we load these first so that we can check them for annotations of type=8

	local XScale=mapWindow:GetWidth()/mapInfo[mapWindow.CurrentMapID][4];
	local YScale=mapWindow:GetHeight()/mapInfo[mapWindow.CurrentMapID][5];

	XScale=1; -- turns out, the parent in Stretchmode=1 automatically takes care of it's children :)
	YScale=1;

	local HFactor=mapInfo[mapWindow.CurrentMapID][6];
	local HOffset=mapInfo[mapWindow.CurrentMapID][7];
	local VFactor=mapInfo[mapWindow.CurrentMapID][8];
	local VOffset=mapInfo[mapWindow.CurrentMapID][9];
	local tmpIndex;
	local tmpIconIndex=0;
	local tmpDescription;
	local tmpShortDescription;
	local pos1;
	local tmpList={};

	for tmpIndex=1,#annotations do
		if annotations[tmpIndex][1]==mapWindow.CurrentMapID then
			tmpIconIndex=tmpIconIndex+1;

			-- check if we need a new control
			if tmpIconIndex>#mapWindow.Map.Icons then
				mapWindow.Map.Icons[tmpIconIndex]=Turbine.UI.Control();
				mapWindow.Map.Icons[tmpIconIndex]:SetParent(mapWindow.Map)
				mapWindow.Map.Icons[tmpIconIndex]:SetBlendMode(Turbine.UI.BlendMode.Overlay);
			end
			mapWindow.Map.Icons[tmpIconIndex]:SetZOrder(100);

			mapWindow.Map.Icons[tmpIconIndex]:SetSize(iconInfo[annotations[tmpIndex][4]][2],iconInfo[annotations[tmpIndex][4]][3]);
			mapWindow.Map.Icons[tmpIconIndex]:SetBackground(iconInfo[annotations[tmpIndex][4]][4]);

			mapWindow.Map.Icons[tmpIconIndex]:SetPosition(math.floor((annotations[tmpIndex][3]*HFactor+HOffset)*XScale)-iconInfo[annotations[tmpIndex][4]][2]/2,math.floor((annotations[tmpIndex][2]*VFactor+VOffset)*YScale)-iconInfo[annotations[tmpIndex][4]][3]/2);
			mapWindow.Map.Icons[tmpIconIndex]:SetVisible(true);
			mapWindow.Map.Icons[tmpIconIndex].AnnotationIndex=tmpIndex;
			mapWindow.Map.Icons[tmpIconIndex].IconIndex=annotations[tmpIndex][4];
			tmpDescription=annotations[tmpIndex][7]; -- use custom text by default
			if tmpDescription==nil or tmpDescription=="" then
				if annotations[tmpIndex][6]~=nil then
					if Resource[language][annotations[tmpIndex][6]]~=nil then
						tmpDescription=Resource[language][annotations[tmpIndex][6]]; -- use translated text if found
					elseif Resource[1][annotations[tmpIndex][6]]~=nil then
						tmpDescription=Resource[1][annotations[tmpIndex][6]]; -- fall back to english if translated text not found
					end
				end
			end
			tmpShortDescription="";
			pos1=string.find (tmpDescription,":",0,true);
			if (pos1~=nil)then
				tmpShortDescription=string.sub(tmpDescription,1,pos1-1);
				tmpDescription=string.sub(tmpDescription,pos1+1);
			end
			mapWindow.Map.Icons[tmpIconIndex].Description=tmpDescription;
			mapWindow.Map.Icons[tmpIconIndex].ShortDescription=tmpShortDescription;
			tmpList[#tmpList+1]={tmpIndex,tmpShortDescription,1};
			if annotations[tmpIndex][2]<0 then
				mapWindow.Map.Icons[tmpIconIndex].MapCoord=tostring(0-annotations[tmpIndex][2]).."S, ";
				tmpList[#tmpList][2]=tmpList[#tmpList][2].."("..tostring(0-annotations[tmpIndex][2]).."S, ";
			else
				mapWindow.Map.Icons[tmpIconIndex].MapCoord=tostring(annotations[tmpIndex][2]).."N, ";
				tmpList[#tmpList][2]=tmpList[#tmpList][2].."("..tostring(annotations[tmpIndex][2]).."N, ";
			end
			if annotations[tmpIndex][3]<0 then
				mapWindow.Map.Icons[tmpIconIndex].MapCoord=mapWindow.Map.Icons[tmpIconIndex].MapCoord..tostring(0-annotations[tmpIndex][3]).."W";
				tmpList[#tmpList][2]=tmpList[#tmpList][2]..tostring(0-annotations[tmpIndex][3]).."W)";
			else
				mapWindow.Map.Icons[tmpIconIndex].MapCoord=mapWindow.Map.Icons[tmpIconIndex].MapCoord..tostring(annotations[tmpIndex][3]).."E";
				tmpList[#tmpList][2]=tmpList[#tmpList][2]..tostring(annotations[tmpIndex][3]).."E)";
			end
			mapWindow.Map.Icons[tmpIconIndex].XYCoord=tostring(math.floor((annotations[tmpIndex][3]*HFactor+HOffset)*XScale))..", "..tostring(math.floor((annotations[tmpIndex][2]*VFactor+VOffset)*YScale));
			if annotations[tmpIndex][5]==8 then
				tmpList[#tmpList][3]=2;
				-- check whether there is a MapPoint set for this point (character must have the map in their inventory)
				local mapFound=false;
				local tmpMPIndex;
				for tmpMPIndex=1,#mapWindow.Map.MapPoints do
					if mapWindow.Map.MapPoints[tmpMPIndex].V==annotations[tmpIndex][2] and mapWindow.Map.MapPoints[tmpMPIndex].H==annotations[tmpIndex][3] then
						mapFound=true;
						break;
					end
				end
				if mapFound then
					mapWindow.Map.Icons[tmpIconIndex]:SetMouseVisible(false)
					mapWindow.Map.Icons[tmpIconIndex].MouseEnter=nil;
					mapWindow.Map.Icons[tmpIconIndex].MouseLeave=nil;
					mapWindow.Map.Icons[tmpIconIndex].MouseHover=nil;
					mapWindow.Map.Icons[tmpIconIndex].MouseClick=nil;

				else
					-- do we want to use the "unearned" icon?
					mapWindow.Map.Icons[tmpIconIndex].IconIndex=13;
					mapWindow.Map.Icons[tmpIconIndex]:SetBackground(iconInfo[13][4]);

					mapWindow.Map.Icons[tmpIconIndex]:SetMouseVisible(true)
					mapWindow.Map.Icons[tmpIconIndex].MouseEnter=function(sender,args)
						sender:SetBackground(iconInfo[sender.IconIndex][5]);
					end
					mapWindow.Map.Icons[tmpIconIndex].MouseLeave=function(sender,args)
						sender:SetBackground(iconInfo[sender.IconIndex][4]);
					end
					mapWindow.Map.Icons[tmpIconIndex].MouseHover=function(sender,args)
						mapWindow.Map.HoverInfo:GetItems():Get(1):SetText(sender.ShortDescription);
						mapWindow.Map.HoverInfo:GetItems():Get(2):SetText(sender.MapCoord);
						mapWindow.Map.HoverInfo:GetItems():Get(3):SetText(sender.XYCoord);
						mapWindow.Map.HoverInfo:GetItems():Get(4):SetText("");
						mapWindow.Map.HoverInfo:GetItems():Get(4):SetEnabled(false);
						mapWindow.Map.HoverInfo:ShowMenu();
					end
					mapWindow.Map.Icons[tmpIconIndex].MouseClick=function(sender,args)
						if mapWindow.PopUp~=nil then
							mapWindow.PopUp:SetVisible(false);
							mapWindow.PopUp=nil; -- dismiss the dialog
						end
						mapWindow.PopUp=PopUpDialog(sender.ShortDescription,sender.Description,4,"OK")
					end
				end
			elseif annotations[tmpIndex][5]==9 then
				tmpList[#tmpList][3]=3; -- npc
				mapWindow.Map.Icons[tmpIconIndex]:SetMouseVisible(false)
				mapWindow.Map.Icons[tmpIconIndex].MouseEnter=nil;
				mapWindow.Map.Icons[tmpIconIndex].MouseLeave=nil;
				mapWindow.Map.Icons[tmpIconIndex].MouseHover=nil;
				mapWindow.Map.Icons[tmpIconIndex].MouseClick=nil;
			else
				mapWindow.Map.Icons[tmpIconIndex]:SetMouseVisible(true)
				mapWindow.Map.Icons[tmpIconIndex].MouseEnter=function(sender,args)
					sender:SetBackground(iconInfo[sender.IconIndex][5]);
				end
				mapWindow.Map.Icons[tmpIconIndex].MouseLeave=function(sender,args)
					sender:SetBackground(iconInfo[sender.IconIndex][4]);
				end
				mapWindow.Map.Icons[tmpIconIndex].MouseHover=function(sender,args)
					mapWindow.Map.HoverInfo:GetItems():Get(1):SetText(sender.ShortDescription);
					mapWindow.Map.HoverInfo:GetItems():Get(2):SetText(sender.MapCoord);
					mapWindow.Map.HoverInfo:GetItems():Get(3):SetText(sender.XYCoord);
					annotWindow:SelectAnnotation(sender.AnnotationIndex);
					mapWindow.Map.HoverInfo:GetItems():Get(4):SetText(Resource[language][155]);
					mapWindow.Map.HoverInfo:GetItems():Get(4):SetEnabled(true);
					mapWindow.Map.HoverInfo:ShowMenu();
				end
				mapWindow.Map.Icons[tmpIconIndex].MouseClick=function(sender,args)
					if mapWindow.PopUp~=nil then
						mapWindow.PopUp:SetVisible(false);
						mapWindow.PopUp=nil; -- dismiss the dialog
					end
					mapWindow.PopUp=PopUpDialog(sender.ShortDescription,sender.Description,4,"OK")
				end
			end
		end
	end	
	local tmpLabel;
	mapWindow.Map.IconList:SetZOrder(101+tmpIconIndex);
	-- sort list by col3 (type 1-3), then col2 (name)
	table.sort(tmpList,function(arg1,arg2) if arg1[3]<arg2[3] then return(true) else if arg1[3]==arg2[3] and arg1[2]<arg2[2] then return(true) end end end);
	local tmpType=0;
	for tmpIndex=1,#tmpList do
		if tmpList[tmpIndex][3]~=tmpType then
			-- insert headers
			tmpType=tmpList[tmpIndex][3];
			tmpLabel=Turbine.UI.Label()
			tmpLabel:SetSize(mapWindow.Map.IconList:GetWidth()-12,20);
			tmpLabel:SetBackColor(backColor);
			tmpLabel:SetBackColor(Turbine.UI.Color(.1,.1,.1));
			tmpLabel:SetForeColor(headingsColor);
			tmpLabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter);
			tmpLabel:SetFont(Turbine.UI.Lotro.Font.TrajanProBold16);
			tmpLabel:SetMultiline(false);
			tmpLabel.AnnotationIndex=0;
			tmpLabel.NormalHeight=20;
			tmpLabel:SetText(Resource[language][136]);
			if tmpType==1 then
				tmpLabel:SetText(Resource[language][158])
			elseif tmpType==2 then
				tmpLabel:SetText(Resource[language][150])
			elseif tmpType==3 then
				tmpLabel:SetText(Resource[language][111])
			end
			mapWindow.Map.IconList:AddItem(tmpLabel);
		end
		tmpLabel=Turbine.UI.Label()
		tmpLabel:SetSize(mapWindow.Map.IconList:GetWidth()-12,18);
		tmpLabel:SetBackColor(backColor);
		tmpLabel:SetForeColor(listTextColor);
		tmpLabel:SetMultiline(false);
		tmpLabel:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleCenter);
		tmpLabel:SetFont(fontFace);
		tmpLabel.NormalHeight=18;
		tmpLabel:SetText(tmpList[tmpIndex][2])
		tmpLabel.AnnotationIndex=tmpList[tmpIndex][1];
		tmpLabel.MouseClick=function(sender,args)
			mapWindow.Map.IconList:SetVisible(false);
			mapWindow.Map.IconSearch:SetVisible(false);
			mapWindow.Map.IconSearch:SetWantsUpdates(false);
			mapWindow.Map.Ping:SetZOrder(101+#mapWindow.Map.Icons);
			mapWindow.Map.Ping:SetPosition(math.floor((annotations[sender.AnnotationIndex][3]*mapInfo[mapWindow.CurrentMapID][6]+mapInfo[mapWindow.CurrentMapID][7])-11.5),math.floor((annotations[sender.AnnotationIndex][2]*mapInfo[mapWindow.CurrentMapID][8]+mapInfo[mapWindow.CurrentMapID][9])-11.5));
			mapWindow.Map.Ping.State=1;
			mapWindow.Map.Ping:SetVisible(true);
			mapWindow.Map.Ping:SetWantsUpdates(true);
		end
		mapWindow.Map.IconList:AddItem(tmpLabel);
	end

	while #mapWindow.Map.Icons>tmpIconIndex do
		-- dispose of any unused controls that we already created
		mapWindow.Map.Icons[#mapWindow.Map.Icons].MouseEnter=nil;
		mapWindow.Map.Icons[#mapWindow.Map.Icons].MouseLeave=nil;
		mapWindow.Map.Icons[#mapWindow.Map.Icons].MouseHover=nil;
		mapWindow.Map.Icons[#mapWindow.Map.Icons].MouseClick=nil;
		mapWindow.Map.Icons[#mapWindow.Map.Icons]:SetVisible(false);-- need to invalidated the area where this control was draw to force a refresh
		table.remove(mapWindow.Map.Icons,#mapWindow.Map.Icons);
	end
end

-- we have to use ItemControls so that we can programatically fill them if the corresponding map is found in the character's inventory
-- definately have to get French and German translations for the map names :(
-- use the "Good" grams map if found, if not, then use the default grams map if found

AddMapPoint=function(mapItem, V, H)
-- I temporarily use 4 controls to cover each map point in a 12x16 area because the mouse events for ItemControls only fire on their borders. As soon as he mouse events are fixed, I will eliminate the excess controls.
	local HFactor=mapInfo[mapWindow.CurrentMapID][6];
	local HOffset=mapInfo[mapWindow.CurrentMapID][7];
	local VFactor=mapInfo[mapWindow.CurrentMapID][8];
	local VOffset=mapInfo[mapWindow.CurrentMapID][9];
	mapWindow.Map.MapPoints[#mapWindow.Map.MapPoints+1]=Turbine.UI.Lotro.ItemControl();
	mapWindow.Map.MapPoints[#mapWindow.Map.MapPoints].V=V;
	mapWindow.Map.MapPoints[#mapWindow.Map.MapPoints].H=H;
	mapWindow.Map.MapPoints[#mapWindow.Map.MapPoints]:SetParent(mapWindow.Map);
--	mapWindow.Map.MapPoints[#mapWindow.Map.MapPoints]:SetPosition(math.floor(H*HFactor+HOffset)-8,math.floor(V*VFactor+VOffset)-8);
--	mapWindow.Map.MapPoints[#mapWindow.Map.MapPoints]:SetSize(16,16);
	mapWindow.Map.MapPoints[#mapWindow.Map.MapPoints]:SetPosition(math.floor(H*HFactor+HOffset)-7,math.floor(V*VFactor+VOffset)-8);
	mapWindow.Map.MapPoints[#mapWindow.Map.MapPoints]:SetSize(3,16);
	mapWindow.Map.MapPoints[#mapWindow.Map.MapPoints]:SetItem(mapItem);
	mapWindow.Map.MapPoints[#mapWindow.Map.MapPoints]:SetVisible(true);
	mapWindow.Map.MapPoints[#mapWindow.Map.MapPoints].MouseDown=function(sender, args)
		if args.Button==2 then
			mapWindow:SetVisible(false);
		end
	end
--	mapWindow.Map.MapPoints[#mapWindow.Map.MapPoints].MouseClick=function(sender, args)
--		if args.Button==2 then
--			mapWindow:SetVisible(false);
--		end
--	end

	mapWindow.Map.MapPoints2[#mapWindow.Map.MapPoints2+1]=Turbine.UI.Lotro.ItemControl();
	mapWindow.Map.MapPoints2[#mapWindow.Map.MapPoints2].V=V;
	mapWindow.Map.MapPoints2[#mapWindow.Map.MapPoints2].H=H;
	mapWindow.Map.MapPoints2[#mapWindow.Map.MapPoints2]:SetParent(mapWindow.Map);
	mapWindow.Map.MapPoints2[#mapWindow.Map.MapPoints2]:SetPosition(math.floor(H*HFactor+HOffset)-5,math.floor(V*VFactor+VOffset)-8);
	mapWindow.Map.MapPoints2[#mapWindow.Map.MapPoints2]:SetSize(3,16);
	mapWindow.Map.MapPoints2[#mapWindow.Map.MapPoints2]:SetItem(mapItem);
	mapWindow.Map.MapPoints2[#mapWindow.Map.MapPoints2]:SetVisible(true);
	mapWindow.Map.MapPoints2[#mapWindow.Map.MapPoints2].MouseDown=function(sender, args)
		if args.Button==2 then
			mapWindow:SetVisible(false);
		end
	end

	mapWindow.Map.MapPoints3[#mapWindow.Map.MapPoints3+1]=Turbine.UI.Lotro.ItemControl();
	mapWindow.Map.MapPoints3[#mapWindow.Map.MapPoints3].V=V;
	mapWindow.Map.MapPoints3[#mapWindow.Map.MapPoints3].H=H;
	mapWindow.Map.MapPoints3[#mapWindow.Map.MapPoints3]:SetParent(mapWindow.Map);
	mapWindow.Map.MapPoints3[#mapWindow.Map.MapPoints3]:SetPosition(math.floor(H*HFactor+HOffset)-3,math.floor(V*VFactor+VOffset)-8);
	mapWindow.Map.MapPoints3[#mapWindow.Map.MapPoints3]:SetSize(3,16);
	mapWindow.Map.MapPoints3[#mapWindow.Map.MapPoints3]:SetItem(mapItem);
	mapWindow.Map.MapPoints3[#mapWindow.Map.MapPoints3]:SetVisible(true);
	mapWindow.Map.MapPoints3[#mapWindow.Map.MapPoints3].MouseDown=function(sender, args)
		if args.Button==2 then
			mapWindow:SetVisible(false);
		end
	end

	mapWindow.Map.MapPoints4[#mapWindow.Map.MapPoints4+1]=Turbine.UI.Lotro.ItemControl();
	mapWindow.Map.MapPoints4[#mapWindow.Map.MapPoints4].V=V;
	mapWindow.Map.MapPoints4[#mapWindow.Map.MapPoints4].H=H;
	mapWindow.Map.MapPoints4[#mapWindow.Map.MapPoints4]:SetParent(mapWindow.Map);
	mapWindow.Map.MapPoints4[#mapWindow.Map.MapPoints4]:SetPosition(math.floor(H*HFactor+HOffset)-1,math.floor(V*VFactor+VOffset)-8);
	mapWindow.Map.MapPoints4[#mapWindow.Map.MapPoints4]:SetSize(3,16);
	mapWindow.Map.MapPoints4[#mapWindow.Map.MapPoints4]:SetItem(mapItem);
	mapWindow.Map.MapPoints4[#mapWindow.Map.MapPoints4]:SetVisible(true);
	mapWindow.Map.MapPoints4[#mapWindow.Map.MapPoints4].MouseDown=function(sender, args)
		if args.Button==2 then
			mapWindow:SetVisible(false);
		end
	end

	mapWindow.Map.MapPoints5[#mapWindow.Map.MapPoints5+1]=Turbine.UI.Lotro.ItemControl();
	mapWindow.Map.MapPoints5[#mapWindow.Map.MapPoints5].V=V;
	mapWindow.Map.MapPoints5[#mapWindow.Map.MapPoints5].H=H;
	mapWindow.Map.MapPoints5[#mapWindow.Map.MapPoints5]:SetParent(mapWindow.Map);
	mapWindow.Map.MapPoints5[#mapWindow.Map.MapPoints5]:SetPosition(math.floor(H*HFactor+HOffset)+1,math.floor(V*VFactor+VOffset)-8);
	mapWindow.Map.MapPoints5[#mapWindow.Map.MapPoints5]:SetSize(3,16);
	mapWindow.Map.MapPoints5[#mapWindow.Map.MapPoints5]:SetItem(mapItem);
	mapWindow.Map.MapPoints5[#mapWindow.Map.MapPoints5]:SetVisible(true);
	mapWindow.Map.MapPoints5[#mapWindow.Map.MapPoints5].MouseDown=function(sender, args)
		if args.Button==2 then
			mapWindow:SetVisible(false);
		end
	end

	mapWindow.Map.MapPoints6[#mapWindow.Map.MapPoints6+1]=Turbine.UI.Lotro.ItemControl();
	mapWindow.Map.MapPoints6[#mapWindow.Map.MapPoints6].V=V;
	mapWindow.Map.MapPoints6[#mapWindow.Map.MapPoints6].H=H;
	mapWindow.Map.MapPoints6[#mapWindow.Map.MapPoints6]:SetParent(mapWindow.Map);
	mapWindow.Map.MapPoints6[#mapWindow.Map.MapPoints6]:SetPosition(math.floor(H*HFactor+HOffset)+3,math.floor(V*VFactor+VOffset)-8);
	mapWindow.Map.MapPoints6[#mapWindow.Map.MapPoints6]:SetSize(3,16);
	mapWindow.Map.MapPoints6[#mapWindow.Map.MapPoints6]:SetItem(mapItem);
	mapWindow.Map.MapPoints6[#mapWindow.Map.MapPoints6]:SetVisible(true);
	mapWindow.Map.MapPoints6[#mapWindow.Map.MapPoints6].MouseDown=function(sender, args)
		if args.Button==2 then
			mapWindow:SetVisible(false);
		end
	end

	mapWindow.Map.MapPoints7[#mapWindow.Map.MapPoints7+1]=Turbine.UI.Lotro.ItemControl();
	mapWindow.Map.MapPoints7[#mapWindow.Map.MapPoints7].V=V;
	mapWindow.Map.MapPoints7[#mapWindow.Map.MapPoints7].H=H;
	mapWindow.Map.MapPoints7[#mapWindow.Map.MapPoints7]:SetParent(mapWindow.Map);
	mapWindow.Map.MapPoints7[#mapWindow.Map.MapPoints7]:SetPosition(math.floor(H*HFactor+HOffset)+5,math.floor(V*VFactor+VOffset)-8);
	mapWindow.Map.MapPoints7[#mapWindow.Map.MapPoints7]:SetSize(3,16);
	mapWindow.Map.MapPoints7[#mapWindow.Map.MapPoints7]:SetItem(mapItem);
	mapWindow.Map.MapPoints7[#mapWindow.Map.MapPoints7]:SetVisible(true);
	mapWindow.Map.MapPoints7[#mapWindow.Map.MapPoints7].MouseDown=function(sender, args)
		if args.Button==2 then
			mapWindow:SetVisible(false);
		end
	end

end
LoadMapsIns=function()
	if mapWindow.CurrentMapID==1 then
		-- search through bags looking for matches to the map names
		local tmpBackpack=localPlayer:GetBackpack();
		local tmpBPIndex;
		local tmpItem;
		local tmpCrudeGrams;
		local tmpCrudeGV;
		local addCrudeGrams=true;
		local addCrudeGV=true;
		for tmpBPIndex=1,tmpBackpack:GetSize() do
			tmpItem=tmpBackpack:GetItem(tmpBPIndex);
			if tmpItem~=nil then
				if tmpItem:GetName()==Resource[language][112] then
					tmpCrudeGrams=tmpItem;
				elseif tmpItem:GetName()==Resource[language][113] then
					addCrudeGrams=false;
					AddMapPoint(tmpItem,-12.3,-20.9);
				elseif tmpItem:GetName()==Resource[language][114] then
					AddMapPoint(tmpItem,-16.1,-20.9);
				elseif tmpItem:GetName()==Resource[language][115] then
					AddMapPoint(tmpItem,-17.1,-18.6);
				elseif tmpItem:GetName()==Resource[language][116] then
					AddMapPoint(tmpItem,-14.6,-15.8);
				elseif tmpItem:GetName()==Resource[language][117] then
					AddMapPoint(tmpItem,-13.4,-12.8);
				elseif tmpItem:GetName()==Resource[language][118] then
					AddMapPoint(tmpItem,-18.3,-18.6);
				elseif tmpItem:GetName()==Resource[language][119] then
					AddMapPoint(tmpItem,-14.2,-21.0);
				elseif tmpItem:GetName()==Resource[language][120] then
					AddMapPoint(tmpItem,-16.0,-17.0);
				elseif tmpItem:GetName()==Resource[language][121] then
					AddMapPoint(tmpItem,-17.8,-15.3);
				elseif tmpItem:GetName()==Resource[language][122] then
					AddMapPoint(tmpItem,-14.7,-14.5);
				elseif tmpItem:GetName()==Resource[language][123] then
					AddMapPoint(tmpItem,-18.9,-17.9);
				elseif tmpItem:GetName()==Resource[language][124] then
					AddMapPoint(tmpItem,-14.5,-19.1);
				elseif tmpItem:GetName()==Resource[language][125] then
					AddMapPoint(tmpItem,-17.9,-18.1);
				elseif tmpItem:GetName()==Resource[language][126] then
					AddMapPoint(tmpItem,-15.7,-14.2);
				elseif tmpItem:GetName()==Resource[language][127] then
					AddMapPoint(tmpItem,-11.5,-19.1);
				elseif tmpItem:GetName()==Resource[language][128] then
					AddMapPoint(tmpItem,-20.3,-16.8);
				elseif tmpItem:GetName()==Resource[language][129] then
					tmpCrudeGV=tmpItem;
				elseif tmpItem:GetName()==Resource[language][130] then
					addCrudeGV=false;
--					AddMapPoint(tmpItem,-20.7,-13.6);
					AddMapPoint(tmpItem,-20.1,-13.6);
				end
			end
		end
		if addCrudeGrams and tmpCrudeGrams~=nil then
			AddMapPoint(tmpCrudeGrams,-12.3,-20.9);
		end
		if addCrudeGV and tmpCrudeGV~=nil then
			AddMapPoint(tmpCrudeGV,-20.7,-13.6);
		end
	else
	-- remove the map points
		while #mapWindow.Map.MapPoints>0 do
			mapWindow.Map.MapPoints[#mapWindow.Map.MapPoints].MouseClick=nil;
			mapWindow.Map.MapPoints[#mapWindow.Map.MapPoints].MouseHover=nil;
			mapWindow.Map.MapPoints[#mapWindow.Map.MapPoints].MouseEnter=nil;
			mapWindow.Map.MapPoints[#mapWindow.Map.MapPoints].MouseLeave=nil;
			mapWindow.Map.MapPoints[#mapWindow.Map.MapPoints].MouseDown=nil;
			table.remove(mapWindow.Map.MapPoints,#mapWindow.Map.MapPoints);

			mapWindow.Map.MapPoints2[#mapWindow.Map.MapPoints2].MouseClick=nil;
			mapWindow.Map.MapPoints2[#mapWindow.Map.MapPoints2].MouseHover=nil;
			mapWindow.Map.MapPoints2[#mapWindow.Map.MapPoints2].MouseEnter=nil;
			mapWindow.Map.MapPoints2[#mapWindow.Map.MapPoints2].MouseLeave=nil;
			mapWindow.Map.MapPoints2[#mapWindow.Map.MapPoints2].MouseDown=nil;
			table.remove(mapWindow.Map.MapPoints2,#mapWindow.Map.MapPoints2);
			mapWindow.Map.MapPoints3[#mapWindow.Map.MapPoints3].MouseClick=nil;
			mapWindow.Map.MapPoints3[#mapWindow.Map.MapPoints3].MouseHover=nil;
			mapWindow.Map.MapPoints3[#mapWindow.Map.MapPoints3].MouseEnter=nil;
			mapWindow.Map.MapPoints3[#mapWindow.Map.MapPoints3].MouseLeave=nil;
			mapWindow.Map.MapPoints3[#mapWindow.Map.MapPoints3].MouseDown=nil;
			table.remove(mapWindow.Map.MapPoints3,#mapWindow.Map.MapPoints3);
			mapWindow.Map.MapPoints4[#mapWindow.Map.MapPoints4].MouseClick=nil;
			mapWindow.Map.MapPoints4[#mapWindow.Map.MapPoints4].MouseHover=nil;
			mapWindow.Map.MapPoints4[#mapWindow.Map.MapPoints4].MouseEnter=nil;
			mapWindow.Map.MapPoints4[#mapWindow.Map.MapPoints4].MouseLeave=nil;
			mapWindow.Map.MapPoints4[#mapWindow.Map.MapPoints4].MouseDown=nil;
			table.remove(mapWindow.Map.MapPoints4,#mapWindow.Map.MapPoints4);

			mapWindow.Map.MapPoints5[#mapWindow.Map.MapPoints5].MouseClick=nil;
			mapWindow.Map.MapPoints5[#mapWindow.Map.MapPoints5].MouseHover=nil;
			mapWindow.Map.MapPoints5[#mapWindow.Map.MapPoints5].MouseEnter=nil;
			mapWindow.Map.MapPoints5[#mapWindow.Map.MapPoints5].MouseLeave=nil;
			mapWindow.Map.MapPoints5[#mapWindow.Map.MapPoints5].MouseDown=nil;
			table.remove(mapWindow.Map.MapPoints5,#mapWindow.Map.MapPoints5);
			mapWindow.Map.MapPoints6[#mapWindow.Map.MapPoints6].MouseClick=nil;
			mapWindow.Map.MapPoints6[#mapWindow.Map.MapPoints6].MouseHover=nil;
			mapWindow.Map.MapPoints6[#mapWindow.Map.MapPoints6].MouseEnter=nil;
			mapWindow.Map.MapPoints6[#mapWindow.Map.MapPoints6].MouseLeave=nil;
			mapWindow.Map.MapPoints6[#mapWindow.Map.MapPoints6].MouseDown=nil;
			table.remove(mapWindow.Map.MapPoints6,#mapWindow.Map.MapPoints6);
			mapWindow.Map.MapPoints7[#mapWindow.Map.MapPoints7].MouseClick=nil;
			mapWindow.Map.MapPoints7[#mapWindow.Map.MapPoints7].MouseHover=nil;
			mapWindow.Map.MapPoints7[#mapWindow.Map.MapPoints7].MouseEnter=nil;
			mapWindow.Map.MapPoints7[#mapWindow.Map.MapPoints7].MouseLeave=nil;
			mapWindow.Map.MapPoints7[#mapWindow.Map.MapPoints7].MouseDown=nil;
			table.remove(mapWindow.Map.MapPoints7,#mapWindow.Map.MapPoints7);
		end
	end
end

function MMWindow:SetLoadMinimized(value)
	if value==nil then value=false end;
	self.LoadMinimized=(value==true);
end
function MMWindow:GetLoadMinimized()
	return (self.LoadMinimized);
end

function MMWindow:SetEnabled(state)
	self.Map.OptionsButton:SetMouseVisible(state);
	self.Map.AnnotationsButton:SetMouseVisible(state);
end

function MMWindow:SetVisible(value)
	Turbine.UI.Lotro.Window.SetVisible( self, value );
end

function MMWindow:Update( sender, args )
	if (Plugins["MoorMap"] ~= nil) and (not self.Loaded) then
		self.Loaded=true;
		Turbine.Shell.WriteLine("MoorMap "..Plugins["MoorMap"]:GetVersion()..Resource[language][13]);
		Plugins["MoorMap"].Unload = function(self,sender,args)
			UnloadPlugin();
		end
	else
		if self.NeedsRefresh then
			self.NeedsRefresh=false;
			RefreshMap();
			self:SetWantsUpdates(false);
		end
	end
end

LoadData();

mapWindow=MMWindow();

if priorDataVersion~=Plugins["MoorMap"]:GetVersion() then
	-- need to read the "default data" and merge with existing data
	import "GaranStuff.MoorMap.Defaults";
	loadDefaults();
end

minWindow=MinimizedWindow();

if mapWindow:GetShowIconWhenMinimized() then
	minWindow:SetSize(32,32);
else
	minWindow:SetSize(0,0);
end

RefreshMap(mapWindow.CurrentMapID);
if mapWindow.LoadMinimized then
	minWindow:SetVisible(true);
else
	mapWindow:SetVisible(true);
end

optWindow=OptionsWindow();
annotWindow=AnnotationsWindow();

optWindow:SetZOrder(mapWindow:GetZOrder()+1);
annotWindow:SetZOrder(mapWindow:GetZOrder()+1);

if dataLoadError then
	-- warn the user and give them the opportunity to exit without saving
end

-- inventory tables have the following structure
-- -- [1] index in backpack
-- -- [2] item durability
-- -- [3] item quality
-- -- [4] item wearstate
-- -- [5] item Name (singular name, we don't have access to plurals yet)
-- -- [6] item quantity
-- -- [7] table of individual quantities (only valid for AllItemArray
-- -- [7][x][1] Character Index - index into characterListArray
-- -- [7][x][2] Quantity
-- -- [8] icon background resource id
-- -- [9] icon image resource id
-- -- [10] icon quality resource id (not currently used)
-- -- [11] icon shadow resource id (not currently used)
-- -- [12] icon underlay image resource id (not currently used)
-- -- [13] image category

--GaranStuff.MoorMap.Table.Dump(Turbine)
--testdialog=PopUpDialog("test1","just testing with a really long message to see how multiple lines are handled. let's see how it handles 3 lines. and now, the coup-de-gras, let's give it 4 lines :)",3,"abcdefghi","just testing","hmmmm")
--testdialog=PopUpDialog("test1","just testing",3,"abcdefghi","just testing","hmmmm")
--testdialog.Buttons[2].Click=function(sender)
--	Turbine.Shell.WriteLine("button 2 pressed");
--	sender:GetParent():SetVisible(false);
--end

